c       stacking tracking information
c
      subroutine cpush(a)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      record /track/ a
      character*70  msg

c
c      call cifDead(a, icon)  ! this is moved to after cpop.
c                             ! so some ptcl may be stored even if 
c                             ! low energy.
c      if(icon .eq. 0) then
         if( Stack_pos .ge. Max_stack_size) then
            write(msg,*) 'stack area full=',Max_stack_size
            call cerrorMsg(msg, 0)
         else
            Stack_pos = Stack_pos + 1
            Stack(Stack_pos) = a
         endif
c      endif
      end
c------------------------------------------------------------
      subroutine cpop(a, remain)
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      record /track/ a

      integer remain
c
c         remain: int. is the number of ptcls remaining unprocessed
c                 including the current one to be processed now.
      if( Stack_pos .le. 0) then
         remain = 0
      else
         remain =  Stack_pos 
         a = Stack(Stack_pos)
         Stack_pos = Stack_pos -1
      endif
      end
c---------------------------------------------------------
      subroutine cinitStack
c         initialize stack.
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      Stack_pos = 0
      end
      subroutine csortStack
c        sort stack dscending order 
      implicit none
#include  "Zmaxdef.h"
#include  "Ztrack.h"
#include  "Zstackv.h"
      real*8 erg(Max_stack_size)
      record /track/ sorted(Max_stack_size)
      integer  idx(Max_stack_size)
      
      integer i, j, k
      record /track/ temp
      do  i = 1,  Stack_pos
         erg(i) = Stack(i).p.fm.p(4)
      enddo
      
      call kqsortd(erg, idx, Stack_pos)
      call ksortinv(idx, Stack_pos)
      
      do i = 1, Stack_pos
         sorted(i) = Stack(idx(i))
      enddo

      do i = 1, Stack_pos
         Stack(i) = sorted(i)
      enddo
      end

      

      
